class filenames:
    #code_dir = 'C://Users/KristiaKarakatsani/Projects/AIWaiter/' #'/Users/kristiakarakatsani/Documents/wAIter/' #
    #code_dir = 'C://Users/JosephinePark/Documents/projects/wAIter/'
    audio_dir = 'audio/'
    menu_dir = 'menus/'
    order_dir = 'orders/'
    code_dir = 'src/'

    audio_file = audio_dir + 'order.wav'
    audio_file2 = audio_dir + 'order_numeric.wav'
    menu_file = menu_dir + 'menu.csv'
    gui_photo = 'photo.ico'
    final_orders = audio_dir + 'orders.csv'
